from datetime import datetime
import json
import pynwb

from element_animal import subject


def subject_to_nwb(session_key):
    subject_query = subject.Subject & session_key
    subject_query = subject_query.join(subject.Subject.Line, left=True)
    subject_query = subject_query.join(subject.Subject.Strain, left=True)
    subject_query = subject_query.join(subject.Subject.Source, left=True)
    subject_info = subject_query.fetch1()

    return pynwb.file.Subject(
        subject_id=subject_info["subject"],
        sex=subject_info["sex"],
        date_of_birth=datetime.combine(
            subject_info["subject_birth_date"],
            datetime.strptime("00:00:00", "%H:%M:%S").time(),
        ),
        description=json.dumps(subject_info, default=str),
        genotype=" x ".join(
            (subject.Line.Allele * subject.Subject.Line & subject_query).fetch("allele")
        ),
    )



import datajoint as dj
import importlib
import inspect


schema = dj.schema()


def activate(
    schema_name, *, create_schema=True, create_tables=True, linking_module=None
):
    """
    activate(schema_name, *, create_schema=True, create_tables=True,
             linking_module=None)
        :param schema_name: schema name on the database server to activate the
                            `subject` element
        :param create_schema: when True (default), create schema in the
                              database if it does not yet exist.
        :param create_tables: when True (default), create tables in the
                              database if they do not yet exist.
        :param linking_module: a module name or a module containing the
         required dependencies to activate the `subject` element:
             Upstream tables:
                + Source: the source of the material/resources
                          (e.g. allele, animal) - typically refers to the
                          vendor (e.g. Jackson Lab - JAX)
                + Lab: the lab for which a particular animal belongs to
                + Protocol: the protocol applicable to a particular animal
                            (e.g. IACUC, IRB)
                + User: the user associated with a particular animal
    """
    if isinstance(linking_module, str):
        linking_module = importlib.import_module(linking_module)
    assert inspect.ismodule(linking_module), (
        "The argument 'dependency' must " + "be a module's name or a module"
    )

    schema.activate(
        schema_name,
        create_schema=create_schema,
        create_tables=create_tables,
        add_objects=linking_module.__dict__,
    )


@schema
class Strain(dj.Lookup):
    definition = """
    # Strain of animal, e.g. C57Bl/6
    strain              : varchar(32)	# abbreviated strain name
    ---
    strain_standard_name  : varchar(32)   # formal name of a strain
    strain_desc=''      : varchar(255)	# description of this strain
    """


@schema
class Allele(dj.Lookup):

    definition = """
    allele                    : varchar(32)  # abbreviated allele name
    ---
    allele_standard_name=''   : varchar(255) # standard name of an allele
    """

    class Source(dj.Part):
        definition = """
        -> master
        ---
        -> Source
        source_identifier=''  : varchar(255) # id inside the line provider
        source_url=''         : varchar(255) # link to the line information
        expression_data_url='': varchar(255) # link to the expression pattern
                                             # from Allen institute brain atlas
        """


@schema
class Line(dj.Lookup):
    definition = """
    line                : varchar(32)  # abbreviated name for the line
    ---
    species=''          : varchar(64)  # Latin name preferred for NWB export
    line_description='' : varchar(2000)
    target_phenotype='' : varchar(255)
    is_active           : boolean	   # whether the line is in active breeding
    """

    class Allele(dj.Part):
        definition = """
        -> master
        -> Allele
        """


@schema
class Subject(dj.Manual):

    definition = """
    # Animal Subject
    subject                 : varchar(8)
    ---
    sex                     : enum('M', 'F', 'U')
    subject_birth_date      : date
    subject_description=''  : varchar(1024)
    """

    class Protocol(dj.Part):
        definition = """
        -> master
        -> Protocol
        """

    class User(dj.Part):
        definition = """
        # Individual responsible for subject management
        -> master
        -> User
        """

    class Line(dj.Part):
        definition = """
        -> master
        ---
        -> Line
        """

    class Strain(dj.Part):
        definition = """
        -> master
        ---
        -> Strain
        """

    class Source(dj.Part):
        definition = """
        -> master
        ---
        -> Source
        """

    class Lab(dj.Part):
        definition = """
        -> master
        -> Lab
        ---
        subject_alias='' : varchar(32) # alias for lab, if different from id
        """


@schema
class SubjectDeath(dj.Manual):
    definition = """
    -> Subject
    ---
    death_date      : date       # death date
    """


@schema
class SubjectCullMethod(dj.Manual):
    definition = """
    -> Subject
    ---
    cull_method:    varchar(255)
    """


@schema
class Zygosity(dj.Manual):
    definition = """
    -> Subject
    -> Allele
    ---
    zygosity : enum("Present", "Absent", "Homozygous", "Heterozygous")
    """



import datajoint as dj
import importlib
import inspect
from . import subject


schema = dj.schema()


def activate(
    genotyping_schema_name,
    subject_schema_name=None,
    create_schema=True,
    create_tables=True,
    linking_module=None,
):
    """
    activate(genotyping_schema_name, subject_schema_name=None,
             create_schema=True, create_tables=True, linking_module=None)
        :param genotyping_schema_name: schema name on the database server to
                                       activate the `genotyping` element
        :param subject_schema_name: schema name on the database server to
                                    activate the `subject` element
        :param create_schema: when True (default), create schema in the
                              database if it does not yet exist.
        :param create_tables: when True (default), create tables in the
                              database if they do not yet exist.
        :param linking_module: a module name or a module containing the
         required dependencies to activate the `genotyping` element:
             Upstream tables:
                + Source: the source of the material/resources
                          (e.g. allele, animal) - typically refers to the
                          vendor (e.g. Jackson Lab - JAX)
                + Lab: the lab for which a particular animal belongs to
                + Protocol: the protocol applicable to a particular animal
                            (e.g. IACUC, IRB)
                + User: the user associated with a particular animal
    """
    if isinstance(linking_module, str):
        linking_module = importlib.import_module(linking_module)
    assert inspect.ismodule(linking_module), (
        "The argument 'dependency' must " + "be a module's name or a module"
    )

    subject.activate(
        subject_schema_name,
        create_schema=create_schema,
        create_tables=create_tables,
        linking_module=linking_module,
    )
    schema.activate(
        genotyping_schema_name,
        create_schema=create_schema,
        create_tables=create_tables,
        add_objects=linking_module.__dict__,
    )


@schema
class Sequence(dj.Lookup):
    definition = """
    sequence            : varchar(32)   # abbreviated sequence name
    ---
    base_pairs=''       : varchar(1024) # base pairs
    sequence_desc=''    : varchar(255)  # description
    """


@schema
class AlleleSequence(dj.Lookup):
    definition = """
    -> subject.Allele
    -> Sequence
    """


@schema
class BreedingPair(dj.Manual):
    definition = """
    -> subject.Line
    breeding_pair           : varchar(32)
    ---
    bp_start_date=null      : date
    bp_end_date=null        : date
    bp_description=''       : varchar(2048)
    """

    class Father(dj.Part):
        definition = """
        -> master
        ---
        -> subject.Subject
        """

    class Mother(dj.Part):
        definition = """
        -> master
        -> subject.Subject
        """


@schema
class Litter(dj.Manual):
    definition = """
    # litter information
    -> BreedingPair
    litter_birth_date       : date
    ---
    num_of_pups             : tinyint
    litter_notes=''         : varchar(255)    # notes
    """


@schema
class Weaning(dj.Manual):
    definition = """
    # weaning information
    -> Litter
    ---
    weaning_date            : date
    num_of_male             : tinyint
    num_of_female           : tinyint
    weaning_notes=''        : varchar(255)
    """


@schema
class SubjectLitter(dj.Manual):
    definition = """
    -> subject.Subject
    ---
    -> Litter
    """


@schema
class Cage(dj.Lookup):
    definition = """
    cage            : varchar(32)   # cage identifying info
    ---
    cage_purpose='' : varchar(128)  # cage purpose
    """


@schema
class SubjectCaging(dj.Manual):

    definition = """
    # record of animal caging
    -> subject.Subject
    caging_datetime     : datetime   # date of cage entry
    ---
    -> Cage
    -> User           # person associated with the cage transfer
    """


@schema
class GenotypeTest(dj.Manual):
    definition = """
    -> subject.Subject
    -> Sequence
    genotype_test_id    : varchar(32)    # identifier of a genotype test
    ---
    test_result         : enum("Present", "Absent")     # test result
    """



#!/usr/bin/env python
from setuptools import setup, find_packages
from os import path

pkg_name = next(p for p in find_packages() if "." not in p)
here = path.abspath(path.dirname(__file__))

with open(path.join(here, "README.md"), "r") as f:
    long_description = f.read()

with open(path.join(here, "requirements.txt")) as f:
    requirements = f.read().splitlines()

with open(path.join(here, pkg_name, "version.py")) as f:
    exec(f.read())

setup(
    name=pkg_name.replace("_", "-"),
    version=__version__,
    description="DataJoint Element for Animal Management",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="DataJoint",
    author_email="info@datajoint.com",
    license="MIT",
    url=f'https://github.com/datajoint/{pkg_name.replace("_", "-")}',
    keywords="neuroscience animal-management datajoint",
    packages=find_packages(exclude=["contrib", "docs", "tests*"]),
    install_requires=requirements,
)
